from enum import Enum


class Attributes(Enum):
    Y_TRUE = "Y_true"
    RUNS = "Rows"
    CADDO_FILE_SETTINGS = "Caddo_file_settings"
    X_RAW = "X_raw"
    Y_RAW = "Y_raw"
    X = "X"
    Y = "Y"
    MODEL = "Model"



import sys

from caddo_tool.settings.settings import Settings


class ModuleLoader:
    def load_model_initializer(self):
        module = self.load_module(Settings.model_initializer_module_path)
        print("Data Loader loaded!")
        return module

    def load_data_preprocessor(self):
        module = self.load_module(Settings.data_preprocessor_module_path)
        print("Data Preprocessor loaded!")
        return module

    def load_model_trainer(self):
        module = self.load_module(Settings.model_trainer_module_path)
        print("Net Trainer loaded!")
        return module

    def load_model_tester(self):
        module = self.load_module(Settings.model_tester_module_path)
        print("Net Tester loaded!")
        return module

    def load_model_evaluator(self):
        module = self.load_module(Settings.model_evaluator_module_path)
        print("Net Evaluator loaded!")
        return module

    def load_module(self, path_to_module):
        __import__(path_to_module)
        return sys.modules[path_to_module]



from copy import deepcopy

from caddo_file_parser.caddo_file_parser import CaddoFileParser
from caddo_file_parser.models.index_set import IndexSet
from pandas import DataFrame

from caddo_tool.modules.attributes import Attributes
from caddo_tool.settings.settings import Settings


class DataLoader:
    def init_metadata(self) -> dict:
        caddo_file = self._get_caddo_file()
        return self._load_metadata(caddo_file)

    def init_attributes_dict(self) -> dict:
        caddo_file = self._get_caddo_file()
        return self._load_attributes(caddo_file)

    def _get_caddo_file(self):
        caddo_file_parser = CaddoFileParser()
        return caddo_file_parser.read_data(Settings.input_data_file)

    def _load_attributes(self, caddo_file) -> dict:
        attributes = dict()
        attributes[Attributes.X_RAW] = self._get_raw_x(caddo_file.data)
        attributes[Attributes.Y_RAW] = self._get_raw_y(caddo_file.data)
        return attributes

    def _get_raw_x(self, data) -> DataFrame:
        return data.filter(regex="^[xX]__", axis=1)

    def _get_raw_y(self, data):
        return data.filter(regex="^[yY]__", axis=1)

    def _load_metadata(self, caddo_file) -> dict:
        metadata = dict()
        metadata[Attributes.RUNS] = caddo_file.runs
        return metadata

    def create_train_attributes(self, attributes, index_set: IndexSet):
        train_attributes = dict()
        train_attributes[Attributes.X] = deepcopy(attributes[Attributes.X].iloc[index_set.train_indexes])
        train_attributes[Attributes.Y] = deepcopy(attributes[Attributes.Y].iloc[index_set.train_indexes])
        train_attributes[Attributes.MODEL] = attributes[Attributes.MODEL]
        return train_attributes

    def create_test_attributes(self, attributes, index_set: IndexSet):
        test_attributes = dict()
        test_attributes[Attributes.X] = deepcopy(attributes[Attributes.X].iloc[index_set.test_indexes])
        test_attributes[Attributes.MODEL] = attributes[Attributes.MODEL]
        return test_attributes

    def enchance_with_proper_responses(self, attributes, index_set: IndexSet, test_attributes):
        test_attributes[Attributes.Y_TRUE] = attributes[Attributes.Y].iloc[index_set.test_indexes]
        return test_attributes



import yaml
from yaml import SafeLoader

from caddo_tool.settings.settings import Settings


class SettingsLoader:
    def __init__(self, settings_path=''):
        self.settings_path = settings_path

    def load(self):
        print("LOADING SETTINGS:")
        self.load_settings_object()
        print()

    def read_settings_file(self):
        with open(self.settings_path) as f:
            data = yaml.load(f, Loader=SafeLoader)
            return data

    def load_settings_object(self):
        settings_file = self.read_settings_file()
        print("Settings:")
        print(yaml.dump(settings_file, default_flow_style=False))
        Settings.input_data_file = settings_file['data']['path']
        Settings.model_initializer_module_path = settings_file['modules']['model_initializer']
        Settings.data_preprocessor_module_path = settings_file['modules']['data_preprocessor']
        Settings.model_trainer_module_path = settings_file['modules']['model_trainer']
        Settings.model_tester_module_path = settings_file['modules']['model_tester']
        Settings.model_evaluator_module_path = settings_file['modules']['model_evaluator']



from caddo_tool.modules.attributes import Attributes
from caddo_tool.modules.module_loader import ModuleLoader
from caddo_tool.settings.settings_loader import SettingsLoader

from caddo_tool.utils.data_loader import DataLoader


class Caddo:
    def __init__(self):
        SettingsLoader('./settings.yaml').load()
        self.date_loader = DataLoader()
        self.model_initializer_module = None
        self.data_preprocessor_module = None
        self.model_trainer_module = None
        self.model_tester_module = None
        self.model_evaluator_module = None
        self.load_modules()
        self.run()

    def load_modules(self):
        print("LOADING MODULES:")
        module_loader = ModuleLoader()
        self.model_initializer_module = module_loader.load_model_initializer()
        self.data_preprocessor_module = module_loader.load_data_preprocessor()
        self.model_trainer_module = module_loader.load_model_trainer()
        self.model_tester_module = module_loader.load_model_tester()
        self.model_evaluator_module = module_loader.load_model_evaluator()
        print()

    def run(self):
        metadata = self.date_loader.init_metadata()
        print("\nPREPARING DATA")
        attributes = self.date_loader.init_attributes_dict()
        self.summarize_raw_attributes(attributes)
        print("\n\n\nPREPROCESSING FILE")
        attributes = self.data_preprocessor_module.preprocess(attributes)
        self.summarize_preprocessed_files(attributes)
        print("\n\n\nInitializing model")
        attributes = self.model_initializer_module.init_model(attributes)
        print("\nBENCHMARKING MODEL\n")
        for run in metadata[Attributes.RUNS]:
            for index_set in run.index_sets:
                print(f"Running benchmark on Run {run.number} index_set {index_set.number}")
                train_attributes = self.date_loader.create_train_attributes(attributes, index_set)
                test_attributes = self.date_loader.create_test_attributes(attributes, index_set)
                print("Training model")
                self.model_trainer_module.train(train_attributes)
                print("Testing model")
                self.model_tester_module.test(test_attributes)
                print("Evaluating model")
                self.model_evaluator_module.evaluate(
                    self.date_loader.enchance_with_proper_responses(attributes, index_set, test_attributes)
                )
                print()

    def summarize_raw_attributes(self, attributes):
        print("Input data summary")
        print("Raw X head:")
        print(attributes[Attributes.X_RAW].head(5))
        print()
        print("Raw Y head:")
        print(attributes[Attributes.Y_RAW].head(5))

    def summarize_preprocessed_files(self, attributes):
        print("Input data summary after preprocessing")
        print("X head:")
        print(attributes[Attributes.X].head(5))
        print()
        print("Y head:")
        print(attributes[Attributes.Y].head(5))


if __name__ == '__main__':
    Caddo()



class Settings:
    input_data_file = ''
    model_initializer_module_path = ''
    data_preprocessor_module_path = ''
    model_trainer_module_path = ''
    model_tester_module_path = ''
    model_evaluator_module_path = ''
